/* Copyright (C) AC SOFTWARE SP. Z O.O.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#define USE_US_TIMER

#include <string.h>
#include <stdio.h>
#include <string.h>

#include <user_interface.h>
#include <espconn.h>
#include <spi_flash.h>
#include <osapi.h>
#include <mem.h>
#include <stdlib.h>
#include <sntp.h>

#include "supla_esp_devconn.h"
#include "supla_esp_cfgmode.h"
#include "supla_esp_gpio.h"
#include "supla_esp_cfg.h"
#include "supla_esp_pwm.h"
#include "supla_esp_hw_timer.h"
#include "supla_ds18b20.h"
#include "supla_dht.h"
#include "supla-dev/srpc.h"
#include "supla-dev/log.h"

#ifdef __FOTA
#include "supla_update.h"
#endif

#define SEND_BUFFER_SIZE1 500
#define SEND_BUFFER_SIZE2 250
#define CVD_MAX_COUNT 4

#define LEAP_YEAR(Y) ( ((1970+Y)>0) && !((1970+Y)%4) && ( ((1970+Y)%100) || !((1970+Y)%400) ) )

static const uint8_t monthDays[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 }; // API starts months from 1, this array starts from 0

typedef struct {
  uint8_t Second;
  uint8_t Minute;
  uint8_t Hour;
  uint8_t Wday;   // day of week, sunday is day 1
  uint8_t Day;
  uint8_t Month;
  uint16_t Year;
  unsigned long Valid;
}TIME_T;

TIME_T rtcTime;
uint32_t myrtc = 0;
uint8_t ntpsync = 0;
uint8_t last_minute = 0;
int init_rtc = 0;
int status_ok = 0;
int sel_prad = 0;
//uint32_t microseconds;
ETSTimer supla_pow_timer_1usec;

typedef struct {
	
	int channel_number;
	ETSTimer timer;
	char value[SUPLA_CHANNELVALUE_SIZE];
	
}channel_value_delayed;

typedef struct {
	ETSTimer supla_devconn_timer1;
	ETSTimer supla_iterate_timer;
	ETSTimer supla_watchdog_timer;
	ETSTimer supla_value_timer;

	// ESPCONN_INPROGRESS fix
	char esp_send_buffer[SEND_BUFFER_SIZE1];
	int esp_send_buffer_len;
	// ---------------------------------------------

	struct espconn ESPConn;
	esp_tcp ESPTCP;
	ip_addr_t ipaddr;

	void *srpc;
	char registered;

	char recvbuff[RECVBUFF_MAXSIZE];
	unsigned int recvbuff_size;
	char laststate[STATE_MAXSIZE];

	char autoconnect;
	unsigned int last_state;
	unsigned int last_relay_state;

	unsigned int last_response;
	unsigned int last_sent;
	int server_activity_timeout;

	uint8 last_wifi_status;

	channel_value_delayed cvd[CVD_MAX_COUNT];
}devconn_params;

typedef struct {
	char esp_send_buffer[SEND_BUFFER_SIZE2];
	int esp_send_buffer_len;

	struct espconn ESPConn;
	esp_tcp ESPTCP;
	ip_addr_t ipaddr;

	void *srpc;
	char registered;

	char recvbuff[RECVBUFF_MAXSIZE2];
	unsigned int recvbuff_size;
	char laststate[STATE_MAXSIZE];

	char autoconnect;
	unsigned int last_state;

	unsigned int last_response;
	unsigned int last_sent;
}devconn_zab_params;

static devconn_zab_params  *devconn_zab = NULL;
static devconn_params      *devconn = NULL;
unsigned int relay_laststate;
unsigned int relay_change = 0;
int ds18b20 = 0;
int	last_temp = -275000;
int	last_humid;
unsigned int timer60;
int snd_zabbix = 0;
unsigned int last_voltage = -1000;
unsigned int last_current;
unsigned int last_power;
unsigned int send_daily_power = 0;
unsigned int last_dif_power = 0;
unsigned int last_dif_current = 0;
unsigned int snd_daily_power = 0;
unsigned int snd_voltage = 0;
unsigned int snd_current = 0;
unsigned int snd_power = 0;
unsigned int error_cnt = 0;
unsigned int send_count = 240;
unsigned int next_t = 1;
uint32_t last_seconds;
uint8_t Last_Day = 0;
int zabbix_client_active = 0; //klient IP aktywny -> =1

#if defined(RGB_CONTROLLER_CHANNEL) \
    || defined(RGBW_CONTROLLER_CHANNEL) \
    || defined(RGBWW_CONTROLLER_CHANNEL) \
    || defined(DIMMER_CHANNEL)


typedef struct {

	char active;
	int counter;

	int ChannelNumber;
	char smoothly;

	int color;
	int dest_color;

	float color_brightness;
	float color_brightness_step;
	float dest_color_brightness;
	
	float brightness;
	float brightness_step;
	float dest_brightness;
		
}devconn_smooth;

devconn_smooth smooth[SMOOTH_MAX_COUNT];

#endif

#if NOSSL == 1
    #define supla_espconn_sent espconn_sent
    #define supla_espconn_disconnect espconn_disconnect
    #define supla_espconn_connect espconn_connect
#else
    #define supla_espconn_sent espconn_secure_sent
	#define _supla_espconn_disconnect espconn_secure_disconnect
	#define supla_espconn_connect espconn_secure_connect
#endif

void DEVCONN_ICACHE_FLASH supla_esp_devconn_timer1_cb(void *timer_arg);
void DEVCONN_ICACHE_FLASH supla_esp_wifi_check_status(char autoconnect);
void DEVCONN_ICACHE_FLASH supla_esp_srpc_free(void);
void DEVCONN_ICACHE_FLASH supla_esp_srpc2_free(void);
void DEVCONN_ICACHE_FLASH supla_esp_devconn_iterate(void *timer_arg);
void DEVCONN_ICACHE_FLASH supla_esp_devconn_zab_resolvandconnect();
void DEVCONN_ICACHE_FLASH supla_get_voltage();
void DEVCONN_ICACHE_FLASH supla_get_current();
void DEVCONN_ICACHE_FLASH supla_get_power();
void ICACHE_FLASH_ATTR sel_pin_voltage();
void DEVCONN_ICACHE_FLASH supla_esp_zabbix_send_restart(void);
void DEVCONN_ICACHE_FLASH supla_esp_zabbix_send_data1(char *item_key, unsigned data1);
void DEVCONN_ICACHE_FLASH supla_esp_zabbix_send_data2(void);

#ifdef POWSENSOR
//================================================================
// System RTC
//================================================================
#ifndef _RTC_H_
#define _RTC_H_

#define NTP_SERVER "pl.pool.ntp.org"

void DEVCONN_ICACHE_FLASH rtc_init(uint8_t timezone);
void DEVCONN_ICACHE_FLASH rtc_timezone(uint8_t timezone);
void DEVCONN_ICACHE_FLASH rtc_second();

#endif

void DEVCONN_ICACHE_FLASH
convertTime()
{

  uint8_t year;
  uint8_t month, monthLength;
  uint32_t time;
  unsigned long days;

  time = (uint32_t) myrtc;
  rtcTime.Second = time % 60;
  time /= 60; // now it is minutes
  rtcTime.Minute = time % 60;
  time /= 60; // now it is hours
  rtcTime.Hour = time % 24;
  time /= 24; // now it is days
  rtcTime.Wday = ((time + 4) % 7) + 1;  // Sunday is day 1

  year = 0;
  days = 0;
  while ((unsigned) (days += (LEAP_YEAR(year) ? 366 : 365)) <= time) {
    year++;
  }
  rtcTime.Year = year; // year is offset from 1970

  days -= LEAP_YEAR(year) ? 366 : 365;
  time -= days; // now it is days in this year, starting at 0

  days = 0;
  month = 0;
  monthLength = 0;
  for (month = 0; month < 12; month++) {
    if (month == 1) { // february
      if (LEAP_YEAR(year)) {
        monthLength = 29;
      }
      else {
        monthLength = 28;
      }
    }
    else {
      monthLength = monthDays[month];
    }

    if (time >= monthLength) {
      time -= monthLength;
    }
    else {
      break;
    }
  }
  rtcTime.Month = month + 1;  // jan is month 1
  rtcTime.Day = time + 1;     // day of month
  rtcTime.Year = rtcTime.Year + 1970;
}

void DEVCONN_ICACHE_FLASH
rtc_second()
{
	if (init_rtc == 1) {
		// NTP Sync every hour at x:0:10
		if (rtcTime.Minute == 0) {
			if ((rtcTime.Second == 10) && !ntpsync) {
				ntpsync = 1;
				myrtc = sntp_get_current_timestamp() -1;
				supla_log(LOG_DEBUG, "RTC: sntp %d, %s \n", myrtc, sntp_get_real_time(myrtc));
			}
			if (rtcTime.Second == 40) ntpsync = 0;
		}
		myrtc++;
		convertTime();
	}
}

void DEVCONN_ICACHE_FLASH
rtc_timezone(uint8_t timezone)
{
	sntp_stop();
//  if (true == sntp_set_timezone(sysCfg.timezone))
	sntp_set_timezone(timezone);
	sntp_init();
	myrtc = sntp_get_current_timestamp();
}

void DEVCONN_ICACHE_FLASH
rtc_init(uint8_t timezone)
{
	sntp_setservername(0, NTP_SERVER);
	sntp_stop();
	sntp_set_timezone(timezone);
	sntp_init();
	myrtc = 0;
	init_rtc = 1;
}
//================================================================
#endif

//================================================================
// System restart
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_system_restart(void) {

    if ( supla_esp_cfgmode_started() == 0 ) {

    	os_timer_disarm(&devconn->supla_watchdog_timer);
    	supla_esp_srpc_free();
    	supla_esp_srpc2_free();

		#ifdef BOARD_GPIO_BEFORE_REBOOT
		supla_esp_board_before_reboot();
		#endif

		//supla_log(LOG_DEBUG, "RESTART");
    	//supla_log(LOG_DEBUG, "Free heap size: %i", system_get_free_heap_size());

    	system_restart();
    }
}

//================================================================
char DEVCONN_ICACHE_FLASH
supla_esp_devconn_update_started(void) {
#ifdef __FOTA
	return supla_esp_update_started();
#else
	return 0;
#endif
}

#pragma GCC diagnostic pop

//================================================================
// SUPLA sprawdzenie czy sa odebrane dane w buforze
//================================================================
int DEVCONN_ICACHE_FLASH
supla_esp_data_read(void *buf, int count, void *dcd) {

	if ( devconn->recvbuff_size > 0 ) {

		count = devconn->recvbuff_size > count ? count : devconn->recvbuff_size;
		os_memcpy(buf, devconn->recvbuff, count);

		if ( count == devconn->recvbuff_size ) {

			devconn->recvbuff_size = 0;

		} else {

			unsigned int a;

			for(a=0;a<devconn->recvbuff_size-count;a++)
				devconn->recvbuff[a] = devconn->recvbuff[count+a];

			devconn->recvbuff_size-=count;
		}

		return count;
	}

	return -1;
}

//================================================================
// Zabbix sprawdzenie czy sa odebrane dane w buforze
//================================================================
int DEVCONN_ICACHE_FLASH
supla_esp_data_zab_read(void *buf, int count, void *dcd) {

	if ( devconn_zab->recvbuff_size > 0 ) {

		count = devconn_zab->recvbuff_size > count ? count : devconn_zab->recvbuff_size;
		os_memcpy(buf, devconn_zab->recvbuff, count);

		if ( count == devconn_zab->recvbuff_size ) {

			devconn_zab->recvbuff_size = 0;

		} else {

			unsigned int a;

			for(a=0;a<devconn_zab->recvbuff_size-count;a++)
				devconn_zab->recvbuff[a] = devconn_zab->recvbuff[count+a];

			devconn_zab->recvbuff_size-=count;
		}

		return count;
	}

	return -1;
}

//================================================================
// SUPLA odczyt danych z sieci do bufora
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_recv_cb(void *arg, char *pdata, unsigned short len) {

	if ( len == 0 || pdata == NULL )
		return;

	//supla_log(LOG_DEBUG, "Supla_recv %i bytes, heap_size: %i", len, system_get_free_heap_size());

	if ( len <= RECVBUFF_MAXSIZE-devconn->recvbuff_size ) {

		os_memcpy(&devconn->recvbuff[devconn->recvbuff_size], pdata, len);
		devconn->recvbuff_size += len;

		supla_esp_devconn_iterate(NULL);

	} else {
		supla_log(LOG_ERR, "Recv1 buffer size exceeded");
	}

}

//================================================================
// Zabbix odczyt danych z sieci do bufora
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_zab_recv_cb(void *arg, char *pdata, unsigned short len) {

	if ( len == 0 || pdata == NULL )
		return;

	//supla_log(LOG_DEBUG, "Zab_recv_cb: %i bytes, heap_size: %i", len, system_get_free_heap_size());

//	os_memcpy(&devconn_zab->recvbuff[devconn_zab->recvbuff_size], pdata, len);
//	supla_log(LOG_DEBUG, "Recv2: %s", &devconn_zab->recvbuff[devconn_zab->recvbuff_size]);
//	if ( len <= RECVBUFF_MAXSIZE2-devconn_zab->recvbuff_size ) {

		//os_memcpy(&devconn_zab->recvbuff[devconn_zab->recvbuff_size], pdata, len);
		//supla_log(LOG_ERR, "Recv2: %s", &devconn_zab->recvbuff[devconn_zab->recvbuff_size]);
		//devconn_zab->recvbuff_size += len;

//	} else {
//		supla_log(LOG_ERR, "Recv2 buffer size exceeded");
//	}

}

//================================================================
// Docz do bufora dane do wysania do serwera ZABBIX
//================================================================
int DEVCONN_ICACHE_FLASH
supla_esp_data_write_zab_append_buffer(void *buf, int count) {

	if ( count > 0 ) {

		if ( devconn_zab->esp_send_buffer_len+count > SEND_BUFFER_SIZE2 ) {

			//supla_log(LOG_ERR, "Send Zabbix buffer size exceeded");
			//supla_esp_devconn_system_restart();
			return -1;

		} else {

			memcpy(&devconn_zab->esp_send_buffer[devconn_zab->esp_send_buffer_len], buf, count);
			devconn_zab->esp_send_buffer_len+=count;

			return 0;

		}
	}

	return 0;
}

//================================================================
// Docz do bufora dane do wysania do serwera SUPLA
//================================================================
int DEVCONN_ICACHE_FLASH
supla_esp_data_write_append_buffer(void *buf, int count) {

	if ( count > 0 ) {

		if ( devconn->esp_send_buffer_len+count > SEND_BUFFER_SIZE1 ) {

			//supla_log(LOG_ERR, "Send Supla buffer size exceeded");
			supla_esp_devconn_system_restart();

			return -1;

		} else {

			memcpy(&devconn->esp_send_buffer[devconn->esp_send_buffer_len], buf, count);
			devconn->esp_send_buffer_len+=count;

			return 0;

		}
	}

	return 0;
}

//================================================================
// Wysanie danych poprzez WIFI do serwera ZABBIX
//   0 : succeed
//  -1 : ESPCONN_MEM - Out of memory
//  -12: ESPCONN_ARG - illegal argument,cant find network transmission according
//================================================================
int DEVCONN_ICACHE_FLASH
supla_esp_data_zab_write(void *buf, int count, void *dcd) {

	int r;

	if ( devconn_zab->esp_send_buffer_len > 0
		 && espconn_sent(&devconn_zab->ESPConn, (unsigned char*)devconn_zab->esp_send_buffer, devconn_zab->esp_send_buffer_len) == 0 ) {

		    //supla_log(LOG_DEBUG, "(1Z)sproto send count: %i", count);
			devconn_zab->esp_send_buffer_len = 0;
			devconn_zab->last_sent = system_get_time();

	};

	if ( devconn_zab->esp_send_buffer_len > 0 ) {
		//supla_log(LOG_DEBUG, "(3Z)Append to bufor");
		return supla_esp_data_write_zab_append_buffer(buf, count);
	}

	if ( count > 0 ) {

		r = espconn_sent(&devconn_zab->ESPConn, buf, count);
		//supla_log(LOG_DEBUG, "(2Z)sproto send count: %i result: %i", count, r);
		//supla_log(LOG_DEBUG,"%s", buf);

		if ( r < 0 ) error_cnt++;
		else error_cnt = 0;
		
		if ( ESPCONN_INPROGRESS == r  ) {
			//supla_log(LOG_DEBUG, "(4Z)Append to bufor");
			return supla_esp_data_write_zab_append_buffer(buf, count);
		} else {

			if ( r == 0 )
				devconn_zab->last_sent = system_get_time();

			return r == 0 ? count : -1;
		}

	}

	return 0;
}

//================================================================
// Wysanie danych poprzez WIFI do serwera SUPLA
//   0 : succeed
//  -1 : ESPCONN_MEM - Out of memory
//  -12: ESPCONN_ARG - illegal argument,cant find network transmission according
//================================================================
int DEVCONN_ICACHE_FLASH
supla_esp_data_write(void *buf, int count, void *dcd) {

	int r;

	if ( devconn->esp_send_buffer_len > 0
		 && supla_espconn_sent(&devconn->ESPConn, (unsigned char*)devconn->esp_send_buffer, devconn->esp_send_buffer_len) == 0 ) {

		    //supla_log(LOG_DEBUG, "(1S)sproto send count: %i", count);
			devconn->esp_send_buffer_len = 0;
			devconn->last_sent = system_get_time();

	};


	if ( devconn->esp_send_buffer_len > 0 ) {
		return supla_esp_data_write_append_buffer(buf, count);
	}

	if ( count > 0 ) {

		r = supla_espconn_sent(&devconn->ESPConn, buf, count);
		//supla_log(LOG_DEBUG, "(2S)sproto send count: %i result: %i", count, r);
		//supla_log(LOG_DEBUG," %s", buf);

		if ( ESPCONN_INPROGRESS == r  ) {
			return supla_esp_data_write_append_buffer(buf, count);
		} else {

			if ( r == 0 )
				devconn->last_sent = system_get_time();

			return r == 0 ? count : -1;
		}

	}

	return 0;
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_set_state(int __pri, const char *message) {

	if ( message == NULL )
		return;

	unsigned char len = strlen(message)+1;

	supla_log(__pri, message);

    if ( len > STATE_MAXSIZE )
    	len = STATE_MAXSIZE;

	os_memcpy(devconn->laststate, message, len);
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_on_version_error(TSDC_SuplaVersionError *version_error) {

	supla_esp_set_state(LOG_ERR, "Protocol version error");
	supla_esp_devconn_stop();
}

//================================================================
// ???
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_send_channel_values_cb(void *ptr) {

	if ( devconn->srpc != NULL
		 && devconn->registered == 1 ) {

		int a;

		for(a=0; a<RS_MAX_COUNT; a++)
			if ( supla_rs_cfg[a].up != NULL
				   && supla_rs_cfg[a].down != NULL
				   && supla_rs_cfg[a].up->channel != 255 ) {

				supla_esp_channel_value_changed(supla_rs_cfg[a].up->channel, ((*supla_rs_cfg[a].position)-100)/100);

			}

		supla_esp_board_send_channel_values_with_delay(devconn->srpc);

	}

}

//================================================================
// ???
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_send_channel_values_with_delay(void) {

	os_timer_disarm(&devconn->supla_value_timer);
	os_timer_setfn(&devconn->supla_value_timer, (os_timer_func_t *)supla_esp_devconn_send_channel_values_cb, NULL);
	os_timer_arm(&devconn->supla_value_timer, 1500, 0);

}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_on_register_result(TSD_SuplaRegisterDeviceResult *register_device_result) {

	char *buff = NULL;

	switch(register_device_result->result_code) {
	case SUPLA_RESULTCODE_BAD_CREDENTIALS:
		supla_esp_set_state(LOG_ERR, "Bad credentials!");
		break;
	case SUPLA_RESULTCODE_TEMPORARILY_UNAVAILABLE:
		supla_esp_set_state(LOG_NOTICE, "Temporarily unavailable!");
		break;

	case SUPLA_RESULTCODE_LOCATION_CONFLICT:
		supla_esp_set_state(LOG_ERR, "Location conflict!");
		break;

	case SUPLA_RESULTCODE_CHANNEL_CONFLICT:
		supla_esp_set_state(LOG_ERR, "Channel conflict!");
		break;

	case SUPLA_RESULTCODE_REGISTRATION_DISABLED:
		supla_esp_set_state(LOG_ERR, "Registration disabled!");
		break;
	case SUPLA_RESULTCODE_AUTHKEY_ERROR:
		supla_esp_set_state(LOG_NOTICE, "Incorrect device AuthKey!");
		break;
	case SUPLA_RESULTCODE_NO_LOCATION_AVAILABLE:
		supla_esp_set_state(LOG_ERR, "No location available!");
		break;
	case SUPLA_RESULTCODE_USER_CONFLICT:
		supla_esp_set_state(LOG_ERR, "User conflict!");
		break;

	case SUPLA_RESULTCODE_TRUE:

		supla_esp_gpio_state_connected();

		devconn->server_activity_timeout = register_device_result->activity_timeout;
		devconn->registered = 1;

		//supla_esp_set_state(LOG_DEBUG, "Registered and ready.");
		status_ok = 1;
		//supla_log(LOG_DEBUG, "Free heap size: %i", system_get_free_heap_size());

		if ( devconn->server_activity_timeout != ACTIVITY_TIMEOUT ) {

			TDCS_SuplaSetActivityTimeout at;
			at.activity_timeout = ACTIVITY_TIMEOUT;
			srpc_dcs_async_set_activity_timeout(devconn->srpc, &at);

		}

		#ifdef __FOTA
		supla_esp_check_updates(devconn->srpc);
		#endif

		supla_esp_devconn_send_channel_values_with_delay();

		#ifdef BOARD_ON_DEVICE_REGISTERED
			BOARD_ON_DEVICE_REGISTERED;
		#endif
		
		#ifdef POWSENSOR
			supla_log(LOG_DEBUG, "supla_pow_setup");
			supla_pow_setup();
		#endif

		supla_esp_zabbix_send_restart();
		return;

	case SUPLA_RESULTCODE_DEVICE_DISABLED:
		supla_esp_set_state(LOG_NOTICE, "Device is disabled!");
		break;

	case SUPLA_RESULTCODE_LOCATION_DISABLED:
		supla_esp_set_state(LOG_NOTICE, "Location is disabled!");
		break;

	case SUPLA_RESULTCODE_DEVICE_LIMITEXCEEDED:
		supla_esp_set_state(LOG_NOTICE, "Device limit exceeded!");
		break;

	case SUPLA_RESULTCODE_GUID_ERROR:
		supla_esp_set_state(LOG_NOTICE, "Incorrect device GUID!");
		break;

	default:

		buff = os_malloc(30);
		ets_snprintf(buff, 30, "Unknown code %i", register_device_result->result_code);
		supla_esp_set_state(LOG_NOTICE, buff);
		os_free(buff);
		buff = NULL;

		break;
	}

	devconn->autoconnect = 0;
	supla_esp_devconn_stop();
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_channel_set_activity_timeout_result(TSDC_SuplaSetActivityTimeoutResult *result) {
	devconn->server_activity_timeout = result->activity_timeout;
}

//================================================================
// Procedura wywoywana przy zmianie wartoci w kanale
// channel_number (np. w/wy przekanika, zmiana temperatury)
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_channel_value_changed(int channel_number, char v) {

	if ( devconn->srpc != NULL
		 && devconn->registered == 1 ) {

		//supla_log(LOG_DEBUG, "DEVCONN: supla_esp_channel_value_changed(%i, %i)", channel_number, v);
		if ( channel_number == 0 ) {
			relay_change = 1;
			relay_laststate = v;
		}

		char value[SUPLA_CHANNELVALUE_SIZE];
		memset(value, 0, SUPLA_CHANNELVALUE_SIZE);
		value[0] = v;

		srpc_ds_async_channel_value_changed(devconn->srpc, channel_number, value);
	}

}

//================================================================
//================================================================
// KOLOROWE TAMY LEDOWE
//================================================================
//================================================================

#if defined(RGBW_CONTROLLER_CHANNEL) \
    || defined(RGBWW_CONTROLLER_CHANNEL) \
	|| defined(RGB_CONTROLLER_CHANNEL) \
    || defined(DIMMER_CHANNEL)

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_channel_rgbw_to_value(char value[SUPLA_CHANNELVALUE_SIZE], int color, char color_brightness, char brightness) {

	memset(value, 0, SUPLA_CHANNELVALUE_SIZE);

	value[0] = brightness;
	value[1] = color_brightness;
	value[2] = (char)((color & 0x000000FF));       // BLUE
	value[3] = (char)((color & 0x0000FF00) >> 8);  // GREEN
	value[4] = (char)((color & 0x00FF0000) >> 16); // RED

}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_channel_value_changed_delayed_cb(void *timer_arg) {
	
	if ( devconn->srpc != NULL
		 && devconn->registered == 1 ) {

		srpc_ds_async_channel_value_changed(devconn->srpc, ((channel_value_delayed*)timer_arg)->channel_number, ((channel_value_delayed*)timer_arg)->value);
	}
	
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_channel_rgbw_value_changed(int channel_number, int color, char color_brightness, char brightness) {

	if ( channel_number >= CVD_MAX_COUNT )
		return;
	
	devconn->cvd[channel_number].channel_number = channel_number;
	
	supla_esp_channel_rgbw_to_value(devconn->cvd[channel_number].value, color, color_brightness, brightness);
	
	os_timer_disarm(&devconn->cvd[channel_number].timer);
	os_timer_setfn(&devconn->cvd[channel_number].timer, (os_timer_func_t *)supla_esp_channel_value_changed_delayed_cb, &devconn->cvd[channel_number]);
	os_timer_arm(&devconn->cvd[channel_number].timer, 1500, 0);

}

#endif
//================================================================
// KOLOROWE TAMY LEDOWE - koniec obsugi 
//================================================================


//================================================================
char DEVCONN_ICACHE_FLASH
_supla_esp_channel_set_value(int port, char v, int channel_number) {

	char _v = v == 1 ? HI_VALUE : LO_VALUE;

	supla_esp_gpio_relay_hi(port, _v, 1);

	_v = supla_esp_gpio_relay_is_hi(port);

	supla_esp_channel_value_changed(channel_number, _v == HI_VALUE ? 1 : 0);

	return (v == 1 ? HI_VALUE : LO_VALUE) == _v;
}

//================================================================
void supla_esp_relay_timer_func(void *timer_arg) {

	_supla_esp_channel_set_value(((supla_relay_cfg_t*)timer_arg)->gpio_id, 0, ((supla_relay_cfg_t*)timer_arg)->channel);

}


//================================================================
//================================================================
// KOLOROWE TAMY LEDOWE
//================================================================
//================================================================
#if defined(RGB_CONTROLLER_CHANNEL) \
    || defined(RGBW_CONTROLLER_CHANNEL) \
    || defined(RGBWW_CONTROLLER_CHANNEL) \
    || defined(DIMMER_CHANNEL)

//================================================================
void DEVCONN_ICACHE_FLASH supla_esp_devconn_smooth_brightness(float *brightness, float *dest_brightness, float *step) {

	if ( *brightness > *dest_brightness ) {

		 *brightness=*brightness - *step;

		 if ( *brightness < 0 )
			 *brightness = 0;

		 if ( *brightness < *dest_brightness )
			 *brightness = *dest_brightness;

	 } else if ( *brightness < *dest_brightness ) {

		 *brightness=*brightness + *step;

		 if ( *brightness > 100 )
			 *brightness = 100;

		 if ( *brightness > *dest_brightness )
			 *brightness = *dest_brightness;
	 }

	*step = (*step) * 1.05;
	
}

//================================================================
hsv DEVCONN_ICACHE_FLASH rgb2hsv(int rgb)
{
    hsv         out;
    double      min, max, delta;

    unsigned char r = (unsigned char)((rgb & 0x00FF0000) >> 16);
    unsigned char g = (unsigned char)((rgb & 0x0000FF00) >> 8);
    unsigned char b = (unsigned char)(rgb & 0x000000FF);

    min = r < g ? r : g;
    min = min  < b ? min  : b;

    max = r > g ? r : g;
    max = max  > b ? max  : b;

    out.v = max;
    delta = max - min;
    if (delta < 0.00001)
    {
        out.s = 0;
        out.h = 0;
        return out;
    }
    if( max > 0.0 ) {
        out.s = (delta / max);
    } else {
        out.s = 0.0;
        out.h = -1;
        return out;
    }
    if( r >= max )
        out.h = ( g - b ) / delta;
    else
    if( g >= max )
        out.h = 2.0 + ( b - r ) / delta;
    else
        out.h = 4.0 + ( r - g ) / delta;

    out.h *= 60.0;

    if( out.h < 0.0 )
        out.h += 360.0;

    return out;
}

//================================================================
int DEVCONN_ICACHE_FLASH hsv2rgb(hsv in)
{
    double      hh, p, q, t, ff;
    long        i;

    unsigned char r,g,b;
    int rgb = 0;

    if(in.s <= 0.0) {       // < is bogus, just shuts up warnings
        r = in.v;
        g = in.v;
        b = in.v;

        rgb = r & 0xFF; rgb<<=8;
        rgb |= g & 0xFF; rgb<<=8;
        rgb |= b & 0xFF;

        return rgb;
    }
    hh = in.h;
    if(hh >= 360.0) hh = 0.0;
    hh /= 60.0;
    i = (long)hh;
    ff = hh - i;
    p = in.v * (1.0 - in.s);
    q = in.v * (1.0 - (in.s * ff));
    t = in.v * (1.0 - (in.s * (1.0 - ff)));

    switch(i) {
    case 0:
        r = in.v; g = t; b = p;
        break;
    case 1:
        r = q; g = in.v; b = p;
        break;
    case 2:
        r = p; g = in.v; b = t;
        break;
    case 3:
        r = p; g = q; b = in.v;
        break;
    case 4:
        r = t; g = p; b = in.v;
        break;
    case 5:
    default:
        r = in.v; g = p; b = q;
        break;
    }

    rgb = r & 0xFF; rgb<<=8;
    rgb |= g & 0xFF; rgb<<=8;
    rgb |= b & 0xFF;

    return rgb;
}

//================================================================
void DEVCONN_ICACHE_FLASH supla_esp_devconn_smooth_cb(devconn_smooth *_smooth) {

	 if ( _smooth->smoothly == 0 || _smooth->counter >= 100 ) {
		 _smooth->color = _smooth->dest_color;
		 _smooth->color_brightness = _smooth->dest_color_brightness;
		 _smooth->brightness = _smooth->dest_brightness;
	 }

	 supla_esp_devconn_smooth_brightness(&_smooth->color_brightness, &_smooth->dest_color_brightness, &_smooth->color_brightness_step);
	 supla_esp_devconn_smooth_brightness(&_smooth->brightness, &_smooth->dest_brightness, &_smooth->brightness_step);

	 if ( _smooth->color != _smooth->dest_color ) {

		 hsv c = rgb2hsv(_smooth->color);
		 hsv dc = rgb2hsv(_smooth->dest_color);

		 c.s = dc.s;
		 c.v = dc.v;

		 if ( c.h < dc.h ) {

			 if ( 360-dc.h+c.h < dc.h-c.h ) {

				 c.h -= 3.6;

				 if ( c.h < 0 )
					 c.h = 359;

			 } else {

				 c.h += 3.6;

				 if ( c.h >= dc.h )
					 c.h = dc.h;
			 }

		 } else if ( c.h > dc.h ) {

			 if ( 360-c.h+dc.h < c.h-dc.h ) {

				 c.h += 3.6;

				 if ( c.h > 359 )
					 c.h = 0;

			 } else {

				 c.h -= 3.6;

				 if ( c.h < dc.h )
					 c.h = dc.h;

			 }

		 }

		 if ( c.h == dc.h ) {
			 _smooth->color = _smooth->dest_color;
		 } else {
			 _smooth->color = hsv2rgb(c);
		 }

	 }

	 supla_esp_board_set_rgbw_value(_smooth->ChannelNumber, &_smooth->color, &_smooth->color_brightness, &_smooth->brightness);
	 _smooth->counter++;

	 if ( _smooth->color == _smooth->dest_color
		  && _smooth->color_brightness == _smooth->dest_color_brightness
		  && _smooth->brightness == _smooth->dest_brightness ) {
		 
		 _smooth->active = 0;
	 }

}

//================================================================
void
_supla_esp_devconn_smooth_cb(void) {

	int a;
	char job = 0;

	for(a=0;a<SMOOTH_MAX_COUNT;a++)
		if ( smooth[a].active == 1 ) {

			job = 1;
			supla_esp_devconn_smooth_cb(&smooth[a]);

		}

	if ( job == 0 ) {
		supla_esp_hw_timer_disarm();
	}

}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_channel_set_rgbw_value(int ChannelNumber, int Color, char ColorBrightness, char Brightness, char smoothly, char send_value_changed) {

	if ( ChannelNumber >= 2 )
		return;
	
	if ( ColorBrightness < 0 ) {
		ColorBrightness = 0;
	} else if ( ColorBrightness > 100 ) {
		ColorBrightness = 100;
	}
	
	if ( Brightness < 0 ) {
		Brightness = 0;
	} else if ( Brightness > 100 ) {
		Brightness = 100;
	}

	supla_esp_hw_timer_disarm();

	devconn_smooth *_smooth = &smooth[ChannelNumber];

	 _smooth->active = 1;
	 _smooth->counter = 0;
	 _smooth->ChannelNumber = ChannelNumber;
	 _smooth->smoothly = smoothly;

	 supla_esp_board_get_rgbw_value(ChannelNumber, &_smooth->color, &_smooth->color_brightness, &_smooth->brightness);
	 	 
	 _smooth->color_brightness_step = abs(_smooth->color_brightness-ColorBrightness)/100.00;
	 _smooth->brightness_step = abs(_smooth->brightness-Brightness)/100.00;
	 
	 _smooth->dest_color = Color;
	 _smooth->dest_color_brightness = ColorBrightness;
	 _smooth->dest_brightness = Brightness;

	 if ( send_value_changed ) {
		 supla_esp_channel_rgbw_value_changed(ChannelNumber, Color, ColorBrightness, Brightness);
	 }

	supla_esp_hw_timer_init(FRC1_SOURCE, 1, _supla_esp_devconn_smooth_cb);
	supla_esp_hw_timer_arm(10000);

}

#endif
//================================================================
// KOLOROWE TAMY LEDOWE - koniec obsugi 
//================================================================


//================================================================
// Ustawienie nowej wartoci dla kanau
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_channel_set_value(TSD_SuplaChannelNewValue *new_value) {

#ifdef BOARD_ON_CHANNEL_VALUE_SET
	 BOARD_ON_CHANNEL_VALUE_SET
#endif /*BOARD_ON_CHANNEL_VALUE_SET*/

#if defined(RGBW_CONTROLLER_CHANNEL) \
	|| defined(RGBWW_CONTROLLER_CHANNEL) \
	|| defined(RGB_CONTROLLER_CHANNEL) \
	|| defined(DIMMER_CHANNEL)

	unsigned char rgb_cn = 255;
	unsigned char dimmer_cn = 255;

    #ifdef RGBW_CONTROLLER_CHANNEL
	rgb_cn = RGBW_CONTROLLER_CHANNEL;
    #endif

	#ifdef RGBWW_CONTROLLER_CHANNEL
	rgb_cn = RGBWW_CONTROLLER_CHANNEL;
	dimmer_cn = RGBWW_CONTROLLER_CHANNEL+1;
	#endif

	#ifdef RGB_CONTROLLER_CHANNEL
	rgb_cn = RGB_CONTROLLER_CHANNEL;
	#endif

	#ifdef DIMMER_CHANNEL
	dimmer_cn = DIMMER_CHANNEL;
	#endif

	if ( new_value->ChannelNumber == rgb_cn
			|| new_value->ChannelNumber == dimmer_cn ) {

		int Color = 0;
		char ColorBrightness = 0;
		char Brightness = 0;

		Brightness = new_value->value[0];
		ColorBrightness = new_value->value[1];

		Color = ((int)new_value->value[4] << 16) & 0x00FF0000; // BLUE
		Color |= ((int)new_value->value[3] << 8) & 0x0000FF00; // GREEN
		Color |= (int)new_value->value[2] & 0x00000FF;         // RED

		if ( Brightness > 100 )
			Brightness = 0;

		if ( ColorBrightness > 100 )
			ColorBrightness = 0;

		if ( new_value->ChannelNumber == rgb_cn ) {

			supla_esp_state.color[0] = Color;
			supla_esp_state.color_brightness[0] = ColorBrightness;
			supla_esp_state.brightness[0] = Brightness;

		} else if ( new_value->ChannelNumber == dimmer_cn ) {

			supla_esp_state.brightness[1] = Brightness;

		}


		supla_esp_channel_set_rgbw_value(new_value->ChannelNumber, Color, ColorBrightness, Brightness, 1, 1);
		supla_esp_save_state(1000);

		return;
	}

#endif


	char v = new_value->value[0];
	int a;
	char Success = 0;

/*
    char buff[200];
    ets_snprintf(buff, 200, "set_value %i,%i,%i", new_value->value[0], new_value->ChannelNumber, new_value->SenderID);
	supla_esp_write_log(buff);
*/
	for(a=0;a<RS_MAX_COUNT;a++)
		if ( supla_rs_cfg[a].up != NULL
			 && supla_rs_cfg[a].down != NULL
			 && supla_rs_cfg[a].up->channel == new_value->ChannelNumber ) {


			int ct = (new_value->DurationMS & 0xFFFF)*100;
			int ot = ((new_value->DurationMS >> 16) & 0xFFFF)*100;

			if ( ct < 0 )
				ct = 0;

			if ( ot < 0 )
				ot = 0;

			if ( ct != supla_esp_cfg.Time2[a]
			     || ot != supla_esp_cfg.Time1[a] ) {

				supla_esp_cfg.Time2[a] = ct;
				supla_esp_cfg.Time1[a] = ot;

				supla_esp_state.rs_position[a] = 0;
				supla_esp_save_state(0);
				supla_esp_cfg_save(&supla_esp_cfg);

				//supla_log(LOG_DEBUG, "Reset RS[%i] position", a);

			}

			//supla_log(LOG_DEBUG, "V=%i", v);

			if ( v >= 10 && v <= 110 ) {

				supla_esp_gpio_rs_add_task(a, v-10);

			} else {

				if ( v == 1 ) {
					supla_esp_gpio_rs_set_relay(&supla_rs_cfg[a], RS_RELAY_DOWN, 1, 1);
				} else if ( v == 2 ) {
					supla_esp_gpio_rs_set_relay(&supla_rs_cfg[a], RS_RELAY_UP, 1, 1);
				} else {
					supla_esp_gpio_rs_set_relay(&supla_rs_cfg[a], RS_RELAY_OFF, 1, 1);
				}

			}

			Success = 1;
			return;
		}

	for(a=0;a<RELAY_MAX_COUNT;a++)
		if ( supla_relay_cfg[a].gpio_id != 255
			 && new_value->ChannelNumber == supla_relay_cfg[a].channel ) {

			Success = _supla_esp_channel_set_value(supla_relay_cfg[a].gpio_id, v, new_value->ChannelNumber);
			break;
		}

	srpc_ds_async_set_channel_result(devconn->srpc, new_value->ChannelNumber, new_value->SenderID, Success);

	if ( v == 1 && new_value->DurationMS > 0 ) {

		for(a=0;a<RELAY_MAX_COUNT;a++)
			if ( supla_relay_cfg[a].gpio_id != 255
				 && new_value->ChannelNumber == supla_relay_cfg[a].channel ) {

				os_timer_disarm(&supla_relay_cfg[a].timer);
				os_timer_setfn(&supla_relay_cfg[a].timer, supla_esp_relay_timer_func, &supla_relay_cfg[a]);
				os_timer_arm (&supla_relay_cfg[a].timer, new_value->DurationMS, false);

				break;
			}

	}
}

//================================================================
// Supla - odebranie i interpretacja polecenia IP
//   DCS - device/client -> server
//   SDC - server -> device/client
//   DS  - device -> server
//   SD  - server -> device
//   CS  - client -> server
//   SC  - server -> client
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_on_remote_call_received(void *_srpc, unsigned int rr_id, unsigned int call_type, void *_dcd, unsigned char proto_version) {

	TsrpcReceivedData rd;
	char result;

	devconn->last_response = system_get_time();

	//supla_log(LOG_DEBUG, "supla call_received");

	if ( SUPLA_RESULT_TRUE == ( result = srpc_getdata(_srpc, &rd, 0)) ) {

		switch(rd.call_type) {
		case SUPLA_SDC_CALL_VERSIONERROR:
			supla_esp_on_version_error(rd.data.sdc_version_error);
			break;
		case SUPLA_SD_CALL_REGISTER_DEVICE_RESULT:
			supla_esp_on_register_result(rd.data.sd_register_device_result);
			break;
		case SUPLA_SD_CALL_CHANNEL_SET_VALUE:
			supla_esp_channel_set_value(rd.data.sd_channel_new_value);
			break;
		case SUPLA_SDC_CALL_SET_ACTIVITY_TIMEOUT_RESULT:
			supla_esp_channel_set_activity_timeout_result(rd.data.sdc_set_activity_timeout_result);
			break;
		#ifdef __FOTA
		case SUPLA_SD_CALL_GET_FIRMWARE_UPDATE_URL_RESULT:
			supla_esp_update_url_result(rd.data.sc_firmware_update_url_result);
			break;
		#endif
		}

		srpc_rd_free(&rd);

	} else if ( result == SUPLA_RESULT_DATA_ERROR ) {

		supla_log(LOG_DEBUG, "SUPLA DATA ERROR!");
	}

}

//================================================================
// Zabbix - odebranie i interpretacja polecenia IP
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_on_remote_call_zab_received(void *_srpc, unsigned int rr_id, unsigned int call_type, void *_dcd, unsigned char proto_version) {

	TsrpcReceivedData rd;
	char result;

	devconn_zab->last_response = system_get_time();

	//supla_log(LOG_DEBUG, "zabbix call_received");

	if ( SUPLA_RESULT_TRUE == ( result = srpc_getdata(_srpc, &rd, 0)) ) {

		/*
		switch(rd.call_type) {
		case SUPLA_SDC_CALL_VERSIONERROR:
			supla_esp_on_version_error(rd.data.sdc_version_error);
			break;
		case SUPLA_SD_CALL_REGISTER_DEVICE_RESULT:
			supla_esp_on_register_result(rd.data.sd_register_device_result);
			break;
		case SUPLA_SD_CALL_CHANNEL_SET_VALUE:
			supla_esp_channel_set_value(rd.data.sd_channel_new_value);
			break;
		case SUPLA_SDC_CALL_SET_ACTIVITY_TIMEOUT_RESULT:
			supla_esp_channel_set_activity_timeout_result(rd.data.sdc_set_activity_timeout_result);
			break;
		#ifdef __FOTA
		case SUPLA_SD_CALL_GET_FIRMWARE_UPDATE_URL_RESULT:
			supla_esp_update_url_result(rd.data.sc_firmware_update_url_result);
			break;
		#endif
		}
		*/

		srpc_rd_free(&rd);

	} else if ( result == SUPLA_RESULT_DATA_ERROR ) {

		supla_log(LOG_DEBUG, "ZABBIX DATA ERROR!");
	}

}

//================================================================
void
supla_esp_devconn_iterate(void *timer_arg) {

	if ( devconn->srpc != NULL ) {

		if ( devconn->registered == 0 ) {
			devconn->registered = -1;

			if ( strlen(supla_esp_cfg.Email) > 0 ) {

				TDS_SuplaRegisterDevice_D srd;
				memset(&srd, 0, sizeof(TDS_SuplaRegisterDevice_C));

				srd.channel_count = 0;
				ets_snprintf(srd.Email, SUPLA_EMAIL_MAXSIZE, "%s", supla_esp_cfg.Email);
				ets_snprintf(srd.ServerName, SUPLA_SERVER_NAME_MAXSIZE, "%s", supla_esp_cfg.Server);

				supla_esp_board_set_device_name(srd.Name, SUPLA_DEVICE_NAME_MAXSIZE);

				strcpy(srd.SoftVer, SUPLA_ESP_SOFTVER);
				os_memcpy(srd.GUID, supla_esp_cfg.GUID, SUPLA_GUID_SIZE);
				os_memcpy(srd.AuthKey, supla_esp_cfg.AuthKey, SUPLA_AUTHKEY_SIZE);

				supla_esp_board_set_channels(srd.channels, &srd.channel_count);

				srpc_ds_async_registerdevice_d(devconn->srpc, &srd);

			} else {

				TDS_SuplaRegisterDevice_C srd;
				memset(&srd, 0, sizeof(TDS_SuplaRegisterDevice_B));

				srd.channel_count = 0;
				srd.LocationID = supla_esp_cfg.LocationID;
				ets_snprintf(srd.LocationPWD, SUPLA_LOCATION_PWD_MAXSIZE, "%s", supla_esp_cfg.LocationPwd);
				ets_snprintf(srd.ServerName, SUPLA_SERVER_NAME_MAXSIZE, "%s", supla_esp_cfg.Server);

				supla_esp_board_set_device_name(srd.Name, SUPLA_DEVICE_NAME_MAXSIZE);

				strcpy(srd.SoftVer, SUPLA_ESP_SOFTVER);
				os_memcpy(srd.GUID, supla_esp_cfg.GUID, SUPLA_GUID_SIZE);

				supla_esp_board_set_channels(srd.channels, &srd.channel_count);

				srpc_ds_async_registerdevice_c(devconn->srpc, &srd);

			}


		};

		supla_esp_data_write(NULL, 0, NULL);

		if( srpc_iterate(devconn->srpc) == SUPLA_RESULT_FALSE ) {
			supla_log(LOG_DEBUG, "iterate fail");
			supla_esp_devconn_system_restart();
		}

	}

}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_srpc_free(void) {

	os_timer_disarm(&devconn->supla_iterate_timer);

	devconn->registered = 0;
	devconn->last_response = 0;
	devconn->last_sent = 0;

	if ( devconn->srpc != NULL ) {
		srpc_free(devconn->srpc);
		devconn->srpc = NULL;
	}
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_srpc2_free(void) {

	//os_timer_disarm(&devconn_zab->supla_iterate_timer);

	devconn_zab->registered = 0;
	devconn_zab->last_response = 0;
	devconn_zab->last_sent = 0;

	if ( devconn_zab->srpc != NULL ) {
		srpc_free(devconn_zab->srpc);
		devconn_zab->srpc = NULL;
	}
}

//================================================================
// Inicjalizacja struktur danych -> client IP Supla
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_srpc_init(void) {
	
	supla_esp_srpc_free();
		
	TsrpcParams srpc_params;
	
	srpc_params_init(&srpc_params);
	srpc_params.data_read = &supla_esp_data_read;
	srpc_params.data_write = &supla_esp_data_write;
	srpc_params.on_remote_call_received = &supla_esp_on_remote_call_received;
	devconn->srpc = srpc_init(&srpc_params);

	os_timer_setfn(&devconn->supla_iterate_timer, (os_timer_func_t *)supla_esp_devconn_iterate, NULL);
	os_timer_arm(&devconn->supla_iterate_timer, 100, 1);

}

//================================================================
// Inicjalizacja struktur danych -> client IP Zabbix
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_srpc2_init(void) {
	
	supla_esp_srpc2_free();
		
	TsrpcParams srpc2_params;
	
	srpc_params_init(&srpc2_params);
	srpc2_params.data_read = &supla_esp_data_zab_read;
	srpc2_params.data_write = &supla_esp_data_zab_write;
	srpc2_params.on_remote_call_received = &supla_esp_on_remote_call_zab_received;
	devconn_zab->srpc = srpc_init(&srpc2_params);
	
}

//================================================================
// Rozczenie z serwerem Supla
//================================================================
void DEVCONN_ICACHE_FLASH
supla_espconn_disconnect(struct espconn *espconn) {
	
	//supla_log(LOG_DEBUG, "Disconnect %i", espconn->state);
	
	if ( espconn->state != ESPCONN_CLOSE
		 && espconn->state != ESPCONN_NONE ) {
		_supla_espconn_disconnect(espconn);
	}
	
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_connect_cb(void *arg) {
	//supla_log(LOG_DEBUG, "devconn_connect_cb\r\n");
	supla_esp_srpc_init();	
	devconn->autoconnect = 1;
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_zab_connect_cb(void *arg) {
	//supla_log(LOG_DEBUG, "devconn_zab_connect_cb\r\n");
	supla_esp_srpc2_init();	
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_disconnect_cb(void *arg){

	//supla_log(LOG_DEBUG, "devconn_disconnect_cb\r\n");
	devconn->autoconnect = 1;
	devconn->last_state = -255;

	 if ( supla_esp_cfgmode_started() == 0
		  && supla_esp_devconn_update_started() == 0 ) {

			supla_esp_srpc_free();
			supla_esp_wifi_check_status(devconn->autoconnect);
	 }
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_zab_disconnect_cb(void *arg){
	
	//supla_log(LOG_DEBUG, "devconn_zab_disconnect_cb\r\n");
	devconn_zab->autoconnect = 1;
	devconn_zab->last_state = -255;

	if ( supla_esp_cfgmode_started() == 0
		&& supla_esp_devconn_update_started() == 0 ) {

			supla_esp_srpc2_free();
	}
	supla_esp_devconn_zab_resolvandconnect();
}

//================================================================
// Poczenie z serwerem SUPLA
//   0  : succeed
//  -1  : ESPCONN_MEM               - Out of memory
//  -3  : ESPCONN_TIMEOUT           - Timeout
//  -4  : ESPCONN_RTE               - Routing Problem
//  -5  : ESPCONN_INPROGRESS        - Operation in progress
//  -7  : ESPCONN_MAXNUM            - Total number exceeds the set maximum
//  -8  : ESPCONN_ABRT              - Connection aborted
//  -9  : ESPCONN_RST               - Connection reset
//  -10 : ESPCONN_CLSD              - Connection closed
//  -11 : ESPCONN_CONN              - Not connected
//  -12 : ESPCONN_ARG               - illegal argument,cant find TCP connection
//  -14 : ESPCONN_IF                - UDP send error
//  -15 : ESPCONN_ISCONN            - Already connected
//  -28 : ESPCONN_HANDSHAKE         - ssl handshake failed
//  -61 : ESPCONN_SSL_INVALID_DATA  - ssl application invalid
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_dns_found_cb(const char *name, ip_addr_t *ip, void *arg) {

    int rel;
	
	if ( ip == NULL ) {
		supla_esp_set_state(LOG_NOTICE, "Domain not found.");
		return;
	}

	supla_log(LOG_DEBUG, "Supla IP: %d.%d.%d.%d",
	*((uint8 *)&ip->addr), *((uint8 *)&ip->addr + 1),
	*((uint8 *)&ip->addr + 2), *((uint8 *)&ip->addr + 3));

	supla_espconn_disconnect(&devconn->ESPConn);

	devconn->ESPConn.proto.tcp = &devconn->ESPTCP;
	devconn->ESPConn.type = ESPCONN_TCP;
	devconn->ESPConn.state = ESPCONN_NONE;
	devconn->ESPConn.link_cnt = 1;

	os_memcpy(devconn->ESPConn.proto.tcp->remote_ip, ip, 4);
	devconn->ESPConn.proto.tcp->local_port = espconn_port();


	#if NOSSL == 1
		devconn->ESPConn.proto.tcp->remote_port = 2015;
	#else
		devconn->ESPConn.proto.tcp->remote_port = 2016;
	
		//espconn_secure_set_size(2, 0);
		//espconn_secure_set_size(1, 3072);
		//supla_log(LOG_DEBUG, "getsize: %i, %i, %i", espconn_secure_get_size(1), espconn_secure_get_size(2), espconn_secure_get_size(3));
	#endif

	//supla_log(LOG_DEBUG, "SUPLA Port - local:%i, remote:%i", devconn->ESPConn.proto.tcp->local_port, devconn->ESPConn.proto.tcp->remote_port);
	espconn_regist_recvcb(&devconn->ESPConn, supla_esp_devconn_recv_cb);
	espconn_regist_connectcb(&devconn->ESPConn, supla_esp_devconn_connect_cb);
	espconn_regist_disconcb(&devconn->ESPConn, supla_esp_devconn_disconnect_cb);

	rel = supla_espconn_connect(&devconn->ESPConn);
	devconn->last_state = rel;
	if (rel == 0)
		supla_log(LOG_DEBUG, "Connected to Supla server (%i)", rel);
    else
		supla_log(LOG_DEBUG, "No connection to Supla server (%i)", rel);
}

//================================================================
// Poczenie z serwerem ZABBIX
//   0  : succeed
//  -1  : ESPCONN_MEM               - Out of memory
//  -3  : ESPCONN_TIMEOUT           - Timeout
//  -4  : ESPCONN_RTE               - Routing Problem
//  -5  : ESPCONN_INPROGRESS        - Operation in progress
//  -7  : ESPCONN_MAXNUM            - Total number exceeds the set maximum
//  -8  : ESPCONN_ABRT              - Connection aborted
//  -9  : ESPCONN_RST               - Connection reset
//  -10 : ESPCONN_CLSD              - Connection closed
//  -11 : ESPCONN_CONN              - Not connected
//  -12 : ESPCONN_ARG               - illegal argument,cant find TCP connection
//  -14 : ESPCONN_IF                - UDP send error
//  -15 : ESPCONN_ISCONN            - Already connected
//  -28 : ESPCONN_HANDSHAKE         - ssl handshake failed
//  -61 : ESPCONN_SSL_INVALID_DATA  - ssl application invalid
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_zab_dns_found_cb(const char *name, ip_addr_t *ip, void *arg) {

    int rel = 1;

	if ( ip == NULL ) {
		supla_esp_set_state(LOG_NOTICE, "Domain not found.");
		return;
	}

	//supla_log(LOG_DEBUG, "Zabbix IP: %d.%d.%d.%d",
	//*((uint8 *)&ip->addr), *((uint8 *)&ip->addr + 1),
	//*((uint8 *)&ip->addr + 2), *((uint8 *)&ip->addr + 3));

	supla_espconn_disconnect(&devconn_zab->ESPConn);

	devconn_zab->ESPConn.proto.tcp = &devconn_zab->ESPTCP;
	devconn_zab->ESPConn.type = ESPCONN_TCP;
	devconn_zab->ESPConn.state = ESPCONN_NONE;
	devconn_zab->ESPConn.link_cnt = 2;

	os_memcpy(devconn_zab->ESPConn.proto.tcp->remote_ip, ip, 4);
	devconn_zab->ESPConn.proto.tcp->local_port = espconn_port();
	devconn_zab->ESPConn.proto.tcp->remote_port = 10051;
	
	//supla_log(LOG_DEBUG, "Zabbix Port - local:%i, remote:%i ", devconn_zab->ESPConn.proto.tcp->local_port, devconn_zab->ESPConn.proto.tcp->remote_port);
	espconn_regist_recvcb(&devconn_zab->ESPConn, supla_esp_devconn_zab_recv_cb);
	espconn_regist_connectcb(&devconn_zab->ESPConn, supla_esp_devconn_zab_connect_cb);
	espconn_regist_disconcb(&devconn_zab->ESPConn, supla_esp_devconn_zab_disconnect_cb);

	rel = espconn_connect(&devconn_zab->ESPConn);
	devconn_zab->last_state = rel;
	if (rel == 0)
		supla_log(LOG_DEBUG, "Connected to Zabbix server (%i)", rel);
    else
		supla_log(LOG_DEBUG, "No connection to Zabbix server (%i)", rel);
}

//================================================================
// PROCEDURA wywoana z "supla_esp_wifi_check_status"
// --------------------------------------------------
// 1. Rozwizanie adresu IP serwera SUPLA
// 2. Odnowienie poczenia z serwerem SUPLA
// 		80.211.244.46 
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_resolvandconnect(void) {

	devconn->autoconnect = 0;
	supla_espconn_disconnect(&devconn->ESPConn);

	uint32_t _ip = ipaddr_addr(supla_esp_cfg.Server);
	
	if ( _ip == -1 ) {
		//supla_log(LOG_DEBUG, "Resolv %s", supla_esp_cfg.Server);
		espconn_gethostbyname(&devconn->ESPConn, supla_esp_cfg.Server, &devconn->ipaddr, supla_esp_devconn_dns_found_cb);
	} else {
		supla_esp_devconn_dns_found_cb(supla_esp_cfg.Server, (ip_addr_t *)&_ip, NULL);
	}
   
}

//================================================================
// PROCEDURA wywoana z "supla_esp_wifi_check_status"
// --------------------------------------------------
// 1. Rozwizanie adresu IP serwera Zabbix
// 2. Odnowienie poczenia z serwerem Zabbix
// 		89.46.196.55 
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_zab_resolvandconnect(void) {

	devconn_zab->autoconnect = 0;
	supla_espconn_disconnect(&devconn_zab->ESPConn);

	uint32_t _ip = ipaddr_addr(supla_esp_cfg.ZabServer);
	
	if ( _ip == -1 ) {
		//supla_log(LOG_DEBUG, "Resolv %s", supla_esp_cfg.ZabServer);
		espconn_gethostbyname(&devconn_zab->ESPConn, supla_esp_cfg.ZabServer, &devconn_zab->ipaddr, supla_esp_devconn_zab_dns_found_cb);
	} else {
		supla_esp_devconn_zab_dns_found_cb(supla_esp_cfg.ZabServer, (ip_addr_t *)&_ip, NULL);
	}
   
}

//================================================================
// Watchdog systemu - restart
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_watchdog_cb(void *timer_arg) {

	 if ( supla_esp_cfgmode_started() == 0
		  && supla_esp_devconn_update_started() == 0 ) {

			unsigned int t = system_get_time();

			if ( t > devconn->last_response && t-devconn->last_response > WATCHDOG_TIMEOUT ) {
				supla_log(LOG_DEBUG, "WATCHDOG TIMEOUT");
				supla_esp_devconn_system_restart();
			}

	 }

}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_before_cfgmode_start(void) {


	#if defined(RGB_CONTROLLER_CHANNEL) \
		|| defined(RGBW_CONTROLLER_CHANNEL) \
		|| defined(RGBWW_CONTROLLER_CHANNEL) \
		|| defined(DIMMER_CHANNEL)

		supla_esp_hw_timer_disarm();

		for(a=0;a<SMOOTH_MAX_COUNT;a++) {
			smooth[a].active = 0;
		}

	#endif

	os_timer_disarm(&devconn->supla_watchdog_timer);


}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_before_update_start(void) {

	os_timer_disarm(&devconn->supla_watchdog_timer);

}

//================================================================
// Inicjalizacja moduu devconn wywoana z user_main
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_init(void) {

    //================================================================
    // Inicjalizacja moduu - serwer Supla
    //================================================================
	supla_log(LOG_DEBUG, "supla_esp_devconn_init");
	if ( devconn == NULL ) {
		devconn = (devconn_params*)malloc(sizeof(devconn_params));
		memset(devconn, 0, sizeof(devconn_params));
	}
	memset(&devconn->ESPConn, 0, sizeof(struct espconn));
	memset(&devconn->ESPTCP, 0, sizeof(esp_tcp));
	
	devconn->last_response = 0;
	devconn->last_sent = 0;
	devconn->autoconnect = 1;
	devconn->last_wifi_status = STATION_GOT_IP+1;
	devconn->last_state = -255;
    
	ets_snprintf(devconn->laststate, STATE_MAXSIZE, "WiFi - Connecting...");
	//sys_wait_for_restart = 0;
    
    //================================================================
    // Inicjalizacja moduu - serwer Zabbix
    //================================================================
	if (supla_esp_cfg.ZabServer[0] > 0) {
		supla_log(LOG_DEBUG, "supla_esp_devconn_zab_init");
		//, "Zabbix host name: %s", supla_esp_cfg.ZabHost);
		if ( devconn_zab == NULL ) {
			devconn_zab = (devconn_zab_params*)malloc(sizeof(devconn_zab_params));
			memset(devconn_zab, 0, sizeof(devconn_zab_params));
		}
		memset(&devconn_zab->ESPConn, 0, sizeof(struct espconn));
		memset(&devconn_zab->ESPTCP, 0, sizeof(esp_tcp));
	
		devconn_zab->last_response = 0;
		devconn_zab->last_sent = 0;
		devconn_zab->autoconnect = 1;
		devconn_zab->last_state = -255;
		timer60 = 10;
		zabbix_client_active = 1;
	}
    //================================================================
	#ifdef POWSENSOR
		microseconds = 0;
		system_timer_reinit();
		os_timer_disarm(&supla_pow_timer_1usec);
		os_timer_setfn(&supla_pow_timer_1usec, (os_timer_func_t *)supla_micros, NULL);
		os_timer_arm_us(&supla_pow_timer_1usec, 5, 1);  // 5 usec
	#endif
	
	os_timer_disarm(&devconn->supla_watchdog_timer);
	os_timer_setfn(&devconn->supla_watchdog_timer, (os_timer_func_t *)supla_esp_devconn_watchdog_cb, NULL);
	os_timer_arm_us(&devconn->supla_watchdog_timer, 1000, 1);

}

//================================================================
//================================================================
// START MODUU DEVCONN (praca na przerwaniach od timera1)
//   - poczenie z sieci WIFI
//   - ustawienie obsugi przerwania na supla_esp_devconn_timer1_cb
//================================================================
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_start(void) {
	
	supla_log(LOG_DEBUG, "supla_esp_devconn_start");
	wifi_station_disconnect();
	
	supla_esp_gpio_state_disconnected();

    struct station_config stationConf;
    memset(&stationConf, 0, sizeof(struct station_config));

    wifi_set_opmode( STATION_MODE );

	#ifdef WIFI_SLEEP_DISABLE
		wifi_set_sleep_type(NONE_SLEEP_T);
	#endif

    os_memcpy(stationConf.ssid, supla_esp_cfg.WIFI_SSID, WIFI_SSID_MAXSIZE);
    os_memcpy(stationConf.password, supla_esp_cfg.WIFI_PWD, WIFI_PWD_MAXSIZE);
   
    stationConf.ssid[31] = 0;
    stationConf.password[63] = 0;
    
    wifi_station_set_config(&stationConf);
    wifi_station_set_auto_connect(1);

    wifi_station_connect();

	os_timer_disarm(&devconn->supla_devconn_timer1);
	os_timer_setfn(&devconn->supla_devconn_timer1, (os_timer_func_t *)supla_esp_devconn_timer1_cb, NULL);
	os_timer_arm(&devconn->supla_devconn_timer1, 1000, 1);
}

//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_stop(void) {
	
	os_timer_disarm(&devconn->supla_devconn_timer1);
	supla_espconn_disconnect(&devconn->ESPConn);
	supla_esp_wifi_check_status(0);
}

//================================================================
char * DEVCONN_ICACHE_FLASH
supla_esp_devconn_laststate(void) {
	return devconn->laststate;
}

//================================================================
char * DEVCONN_ICACHE_FLASH
supla_esp_devconn_zab_laststate(void) {
	return devconn_zab->laststate;
}

//================================================================
// PROCEDURA wywoana z przerwania timera1
// ---------------------------------------
// Pobranie statusu poczenia WIFI
//  i ew. odnowienie poczenia z routerem WIFI oraz z serwerem SUPLA
// 		0 : STATION_IDLE
// 		1 : STATION_CONNECTING
// 		2 : STATION_WRONG_PASSWORD
// 		3 : STATION_NO_AP_FOUND
// 		4 : STATION_CONNECT_FAIL
// 		5 : STATION_GOT_IP
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_wifi_check_status(char autoconnect) {

	uint8 status = wifi_station_get_connect_status();
	//supla_log(LOG_DEBUG, "WiFi Status: %i", status);
	#ifdef POWSENSOR
		if (( status == 5 ) && ( next_t == 1)) {
			next_t--;
			rtc_init(1);
			rtc_second();
			if ( rtcTime.Minute < 15) last_minute = 15;
			else if ( rtcTime.Minute < 30) last_minute = 30;
			else if ( rtcTime.Minute < 45) last_minute = 45;
			else if ( rtcTime.Minute > 45) last_minute = 0;
			//supla_log(LOG_DEBUG, "Next send at: %i (minute)", last_minute);
		}
	#endif

	if ( status != devconn->last_wifi_status )
		supla_log(LOG_DEBUG, "WiFi Status: %i", status);

	devconn->last_wifi_status = status;
		
	if ( STATION_GOT_IP == status ) {
		#ifdef POWSENSOR
			supla_esp_gpio_set_hi(LED_RED_PORT, 1);
		#endif

		if ( devconn->srpc == NULL ) {
			supla_esp_gpio_state_ipreceived();

			//--------------------------------------
			//Odnowienie poczenia z serwerami
			//--------------------------------------
			if ( autoconnect == 1 ) {
				supla_esp_devconn_resolvandconnect();
				if ( zabbix_client_active == 1 ) {
					//timer60 = 12;
					//supla_log(LOG_DEBUG, "supla_esp_devconn_zab_resolvandconnect 1");
					supla_esp_devconn_zab_resolvandconnect();
				}
			}
			//--------------------------------------
		}
	} else {
		switch(status) {

			case STATION_NO_AP_FOUND:
				supla_esp_set_state(LOG_NOTICE, "SSID Not found");
				break;
			case STATION_WRONG_PASSWORD:
				supla_esp_set_state(LOG_NOTICE, "WiFi - Wrong password");
				break;
		}
		supla_esp_gpio_state_disconnected();
	}

}

//================================================================
// Wysanie danych do serwera ZABBIX
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_zabbix_send_data1(char *item_key, unsigned data1) {

	if ( zabbix_client_active == 1 ) {
		if ( last_voltage != -1000) {
			char buff[160];
			char host[30];

			if ( supla_esp_cfg.ZabHost[0] == 0 )
				strcpy(host, "sonoff");
			else
				strcpy(host, supla_esp_cfg.ZabHost);
	
			ets_snprintf(buff, 160, "{\"request\":\"sender data\",\"data\":[{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}]}",host,item_key,data1);
			supla_log(LOG_DEBUG, "%s", buff);
			supla_esp_data_zab_write(buff, os_strlen(buff), NULL);
		}
	}
}

//================================================================
// Wysanie danych do serwera ZABBIX
//================================================================
#ifdef POWSENSOR	
void DEVCONN_ICACHE_FLASH
supla_esp_zabbix_send_data2(void) {

	if ( zabbix_client_active == 1 ) {
		if ( last_voltage != -1000) {
			char buff[250];
			char host[30];
			char item_key1[8];
			char item_key2[10];
			char item_key3[10];
			char item_key4[8];

			if ( supla_esp_cfg.ZabHost[0] == 0 )
				strcpy(host, "sonoff");
			else
				strcpy(host, supla_esp_cfg.ZabHost);
	
			strcpy(item_key1, "state");
			strcpy(item_key2, "voltage");
			strcpy(item_key3, "current");
			strcpy(item_key4, "power");
			ets_snprintf(buff, 250, "{\"request\":\"sender data\",\"data\":[{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"},{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}, \
			{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"},{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}]}", \
			host, item_key1, relay_laststate, host, item_key2, last_voltage, host, item_key3, last_current, host, item_key4, last_power);
	
			supla_esp_data_zab_write(buff, os_strlen(buff), NULL);
		}
	}
}
#endif

//================================================================
// Wysanie informacji o restarcie moduu do serwera ZABBIX
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_zabbix_send_restart(void) {

	if ( zabbix_client_active == 1 ) {
		char buff[160];
		char host[30];
		char hostname[12];
		char item_key[16];

		strcpy(item_key, "hostname");
		strcpy(hostname, "sonoff_rst");
	
		if ( supla_esp_cfg.ZabHost[0] == 0 )
			strcpy(host, "sonoff");
		else
			strcpy(host, supla_esp_cfg.ZabHost);
	
		ets_snprintf(buff, 160, "{\"request\":\"sender data\",\"data\":[{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%s\"}]}",hostname,item_key,host);
		supla_log(LOG_DEBUG, "%s", buff);
		supla_esp_data_zab_write(buff, os_strlen(buff), NULL);
	}
}

#ifndef POWSENSOR
//================================================================
// Wysanie danych do serwera ZABBIX (SONOFF TH16)
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_zabbix_send_data(void) {

    char buff[300];
	char host[30];
	char item_key1[16];

	strcpy(item_key1, "state");
	if ( supla_esp_cfg.ZabHost[0] == 0 )
		strcpy(host, "sonoff");
	else
		strcpy(host, supla_esp_cfg.ZabHost);
	
	ets_snprintf(buff, 160, "{\"request\":\"sender data\",\"data\":[{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}]}",host,item_key1,relay_laststate);

	//------------------------------------
	// Aktywny czujnik DS18b20
	//------------------------------------
    #if defined(TEMPERATURE_CHANNEL)
	if (ds18b20 == 1) {
		char item_key2[16];
		if (last_temp > -275000) {
			strcpy(item_key2, "tempr");
			ets_snprintf(buff, 200, "{\"request\":\"sender data\",\"data\":[ \
			{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"},{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}]}",	host, item_key1, relay_laststate, host, item_key2, last_temp);
		}
		else
			ets_snprintf(buff, 160, "{\"request\":\"sender data\",\"data\":[{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}]}", host, item_key1, relay_laststate);
	}
	#endif
	//------------------------------------

	//------------------------------------
	// Aktywny czujnik AM2301
	//------------------------------------
	#if defined(TEMPERATURE_HUMIDITY_CHANNEL)
	if (ds18b20 == 0) {
		char item_key3[16];
		char item_key4[16];
		if (last_temp > -275000) {
			strcpy(item_key3, "tempr");
			strcpy(item_key4, "humid");
			ets_snprintf(buff, 280, "{\"request\":\"sender data\",\"data\":[ \
			{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}, \
			{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}, \
			{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}]}", \
			host, item_key1, relay_laststate, host, item_key3, last_temp, host, item_key4, last_humid);
		}
		else
			ets_snprintf(buff, 160, "{\"request\":\"sender data\",\"data\":[{\"host\":\"%s\",\"key\":\"%s\",\"value\":\"%i\"}]}", host, item_key1, relay_laststate);
	}	
	#endif
	//------------------------------------

	//supla_log(LOG_DEBUG, "%s", buff);
	supla_esp_data_zab_write(buff, os_strlen(buff), NULL);
}
#endif

//================================================================
// OBSUGA PRZERWANIA os_timer_arm
//	   Timer: 1 sekunda
//================================================================
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_timer1_cb(void *timer_arg) {

	#ifdef POWSENSOR	
		rtc_second();
		//-----------------------------------------------
		// Wysanie dziennego zuycia energii o godzinie 23.50 (wynik podzieli przez 100)
		if (( rtcTime.Hour == 23 ) && ( rtcTime.Minute == 50 ) && ( rtcTime.Day != Last_Day )) {
			snd_daily_power = (unsigned int)(supla_esp_state.daily_power/36000);
			supla_log(LOG_DEBUG, "Daily Power: %i", snd_daily_power);
			supla_esp_zabbix_send_data1("dailypower", snd_daily_power);
			supla_esp_state.daily_power = 0;
			Last_Day = rtcTime.Day;
		}
		//if (snd_zabbix > 0) supla_esp_zabbix_send_data();
		if ( send_count > 0 ) {
			send_count--;
			if (send_count == 180 )      snd_voltage = 1;
			else if (send_count == 120 ) snd_current = 1;
			else if (send_count == 60 ) snd_power = 1;
		}
		if ( send_count == 0 ) send_count = 240;
		//if ( snd_current > 0 ) snd_current--;
		//if ( snd_power > 0 )   snd_power--;
	#endif	

	//supla_log(LOG_DEBUG, "wifi check status");
	supla_esp_wifi_check_status(devconn->autoconnect);

	//===============================
	if ( error_cnt > 10 ) {
		supla_log(LOG_DEBUG, "Error_cnt: %i - restart", error_cnt);
		supla_esp_devconn_system_restart();
	}
	//===============================

	unsigned int t1;
	unsigned int t2;
	unsigned int t3;

	//supla_log(LOG_DEBUG, "Free heap size: %i", system_get_free_heap_size());
	if ( zabbix_client_active == 1 ) {
		if (relay_change == 1) {
			relay_change = 0;
			supla_esp_zabbix_send_data2();
			//supla_log(LOG_DEBUG, "Relay State: %i", relay_laststate);
			send_count = 240;
		}
	}

	#ifdef POWSENSOR
		if (status_ok == 1) {
		//------------------------------------------------------------
			//sel_pin_voltage();	 // w nastpnym cyklu pomiar napicia
			//supla_log(LOG_DEBUG, "Status1: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
			if (sel_nap < licznik_przel) {
				sel_nap++;
				if (sel_nap == 3) {
					licznik_pomiarow = 201;
					pomiar = 1;
					//supla_log(LOG_DEBUG, "Start pomiaru");
					//supla_log(LOG_DEBUG, "Status2: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
				}
				else {
					//supla_log(LOG_DEBUG, "Status3: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
					if ((sel_nap == 5) || (sel_nap == 6)){
						if (od_ready == 1) {
							if (nap_prad == 0) {
								//supla_log(LOG_DEBUG, "Status4: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
								supla_get_voltage(); // odczyt zmierzonego napicia
								od_ready = 0;
								nap_prad = 1;
								sel_pin_current();	 // w nastpnym cyklu pomiar prdu
								//supla_log(LOG_DEBUG, "sel_pin_current");
							}
							else if (nap_prad == 1) {
								//supla_log(LOG_DEBUG, "Status5: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
								supla_get_current(); // odczyt zmierzonego prdu
								od_ready = 0;
								nap_prad = 2;
							}
							else if (nap_prad == 2) {
								//supla_log(LOG_DEBUG, "Status6: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
								supla_get_power(); // odczyt zmierzonej mocy
								od_ready = 0;
								nap_prad = 0;
								sel_pin_voltage();	 // w nastpnym cyklu pomiar napicia
								//supla_log(LOG_DEBUG, "sel_pin_voltage");
							}
						}
					}
				}
			}
			else {
				//supla_log(LOG_DEBUG, "Status7: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
				if (sel_nap >= licznik_przel) {
					//supla_log(LOG_DEBUG, "Status8: microseconds:%i , voltage_time:%i , sel_nap:%i , od_ready:%i",microseconds,voltage_time,sel_nap,od_ready);
					od_ready  = 0;
					sel_nap = 0;
				}	
			}
		//------------------------------------------------------------
		}
	#endif
	
	if ( devconn->registered == 1
		 && devconn->server_activity_timeout > 0
		 && devconn->srpc != NULL ) {

		    t1 = system_get_time();
		    t2 = abs((t1-devconn->last_sent)/1000000);
		    t3 = abs((t1-devconn->last_response)/1000000);

		    if ( t3 >= (devconn->server_activity_timeout+10) ) {

		    	//supla_log(LOG_DEBUG, "Activity timeout %i, %i, %i, %i",  t1, devconn->last_response, (t1-devconn->last_response)/1000000, devconn->server_activity_timeout+10);

		    	supla_esp_srpc_free();
		    	supla_esp_srpc2_free();
		    	supla_esp_wifi_check_status(devconn->autoconnect);

		    } else if ( ( t2 >= (devconn->server_activity_timeout-5)
		    		      && t2 <= devconn->server_activity_timeout )
		    		    || ( t3 >= (devconn->server_activity_timeout-5)
		    		         && t3 <= devconn->server_activity_timeout ) ) {

		    	//supla_log(LOG_DEBUG, "PING");
		    	//system_print_meminfo();
				
				//----------------------------------------------------------------------
				// Odnowienie poczenia z serwerem Zabbix
				//----------------------------------------------------------------------
				if ( zabbix_client_active == 1 ) {
					if ( devconn_zab->last_state == -255 ){
						supla_esp_devconn_zab_resolvandconnect();
					}
				
					#ifdef POWSENSOR
						if ( rtcTime.Minute == last_minute ) {
							//supla_log(LOG_DEBUG, "supla zabbix send data1");
							supla_esp_zabbix_send_data2();
							if ( last_minute == 0 ) last_minute = 15;
							else if ( last_minute == 15 ) last_minute = 30;
							else if ( last_minute == 30 ) last_minute = 45;
							else if ( last_minute == 45 ) last_minute = 0;
							send_count = 240;
						}
						#ifndef POWSENSOR
						else if (snd_zabbix > 0) {
							supla_esp_zabbix_send_data2();
						}
						#endif
					#else
						#ifndef POWSENSOR
						if ( timer60 > 0)
							timer60 = timer60 - 1;
						else {
							timer60 = 143;   // 143->15 minut
							supla_esp_zabbix_send_data();
						}
						#endif
					#endif
				}
				//----------------------------------------------------------------------

				srpc_dcs_async_ping_server(devconn->srpc);
				
			}
	}
}

//================================================================
// Odczyt temperatury, cinienia
//================================================================
#if defined(TEMPERATURE_CHANNEL) || defined(TEMPERATURE_HUMIDITY_CHANNEL)
void DEVCONN_ICACHE_FLASH
supla_esp_devconn_on_temp_humidity_changed(char humidity) {

	double temp;
	int last_temp1;
	
	if ( devconn->srpc != NULL
		 && devconn->registered == 1 ) {

		char value[SUPLA_CHANNELVALUE_SIZE];

        #if defined(TEMPERATURE_CHANNEL)
		    memset(value, 0, sizeof(SUPLA_CHANNELVALUE_SIZE));
			supla_get_temperature(value);
			srpc_ds_async_channel_value_changed(devconn->srpc, TEMPERATURE_CHANNEL, value);
			memcpy(&temp, value, sizeof(double));
			last_temp1 = (int)(temp * 1000.0);
			if (last_temp1 > -275000) {
				ds18b20 = 1;
				last_temp = last_temp1;
				//supla_log(LOG_DEBUG, "Temp_DS18: %i", last_temp);
			}
		#endif

        #if defined(TEMPERATURE_HUMIDITY_CHANNEL)
		int temp1;
		
		if (ds18b20 == 0) {
			memset(value, 0, sizeof(SUPLA_CHANNELVALUE_SIZE));
			supla_get_temp_and_humidity(value);
			memcpy(&temp1, &value[0], sizeof(int));
			last_temp = (int)(temp1);
			supla_log(LOG_DEBUG, "Temp_DHT: %i", last_temp);
			memcpy(&temp1, &value[4], sizeof(int));
			last_humid = (int)(temp1);
			supla_log(LOG_DEBUG, "Wilg_DHT: %i", last_humid);
			srpc_ds_async_channel_value_changed(devconn->srpc, TEMPERATURE_HUMIDITY_CHANNEL, value);
		}
		#endif

	}
}
#endif
//================================================================

//================================================================
// Odczyt parametrw pobieranej mocy
//================================================================
#ifdef POWSENSOR
void DEVCONN_ICACHE_FLASH
supla_get_voltage() {

	unsigned int napiecie;
	char value[SUPLA_CHANNELVALUE_SIZE];

    memset(value, 0, sizeof(SUPLA_CHANNELVALUE_SIZE));
	supla_getVoltage(value);
	memcpy(&napiecie, value, sizeof(uint32_t));
	last_voltage = (int)(napiecie);
	if (last_voltage > 0) 					// ZABBIX: wynik podzieli przez 10
		last_voltage = (int)(404850/last_voltage);
	else
		last_voltage = 0;
	last_voltage =  last_voltage * voltage_multiplier;
	supla_log(LOG_DEBUG, "Voltage: %i", last_voltage);
	if (snd_voltage == 1) {
		supla_esp_zabbix_send_data2();
		snd_voltage = 0;
	}
	voltage_time = 0;
	memcpy(value, &last_voltage, 4);
	srpc_ds_async_channel_value_changed(devconn->srpc, VOLTAGE_CHANNEL, value);
}

void DEVCONN_ICACHE_FLASH
supla_get_current() {

	unsigned int prad;
	unsigned int current_difference = 0;
	char value[SUPLA_CHANNELVALUE_SIZE];

    memset(value, 0, sizeof(SUPLA_CHANNELVALUE_SIZE));
	supla_getVoltage(value);
	memcpy(&prad, value, sizeof(uint32_t));
	last_current = (int)(prad);
	if (last_current > 0)					// ZABBIX: wynik podzieli przez 100
		last_current = (int)(1450000/last_current);
	else
		last_current = 0;
	last_current = last_current * current_multiplier;
	if (last_dif_current > 0) {
		if ( last_current >= last_dif_current )  current_difference = (int)(((last_current - last_dif_current)*100)/last_dif_current);
		else  current_difference = (int)(((last_dif_current - last_current)*100)/last_dif_current);
		if ( current_difference >= 10 ) {
			supla_esp_zabbix_send_data2();
		}
	}
	else {
		if (last_power > 20) {
			supla_esp_zabbix_send_data2();
		}
	}
	supla_log(LOG_DEBUG, "Current: %i", last_current);
	last_dif_current = last_current;
	if ( snd_current == 1) {
		supla_esp_zabbix_send_data2();
		snd_current = 0;
	}
	voltage_time = 0;
	memcpy(value, &last_current, 4);
	srpc_ds_async_channel_value_changed(devconn->srpc, CURRENT_CHANNEL, value);
}

void DEVCONN_ICACHE_FLASH
supla_get_power() {

	unsigned int power;
    uint32_t sekundy;
    uint32_t time_difference;
	unsigned int power_difference = 0;
	char value[SUPLA_CHANNELVALUE_SIZE];

    memset(value, 0, sizeof(SUPLA_CHANNELVALUE_SIZE));
	supla_getVoltage(value);
	memcpy(&power, value, sizeof(uint32_t));
	last_power = (int)(power);
	sekundy = (uint32_t)(sntp_get_current_timestamp());
	time_difference = sekundy - last_seconds;
	if (last_power > 0)	{					// ZABBIX: (wynik x 1)
		last_power = (int)(1034682/last_power);
		// Aktualizacja licznika dziennej energii
		supla_esp_state.daily_power = supla_esp_state.daily_power + last_power*time_difference;
	}
	else last_power = 0;
	last_power =  last_power * power_multiplier;
	if (last_dif_power > 0) {
		if ( last_power >= last_dif_power )  power_difference = (int)(((last_power - last_dif_power)*100)/last_dif_power);
		else  power_difference = (int)(((last_dif_power - last_power)*100)/last_dif_power);
		if ( power_difference >= 10 ) {
			supla_esp_zabbix_send_data2();
		}
	}
	else {
		if (last_power > 20) {
			supla_esp_zabbix_send_data2();
		}
	}
	//-----------------------------------------------
	//supla_log(LOG_DEBUG, "power_difference: %i, last_power: %i W, last_dif_power: %i W, interval: %i sec", power_difference, last_power, last_dif_power, time_difference);
	supla_log(LOG_DEBUG, "=== Daily Power: %u, Next time: %i", supla_esp_state.daily_power, last_minute);
	//-----------------------------------------------
	if (snd_power == 1) {
		supla_esp_zabbix_send_data2();
		snd_power = 0;
	}
	last_dif_power = last_power;
	supla_log(LOG_DEBUG, "Power: %i", last_power);
	last_seconds = sekundy;
	voltage_time = 0;
	memcpy(value, &last_power, 4);
	srpc_ds_async_channel_value_changed(devconn->srpc, POWER_CHANNEL, value);
}

#endif
//================================================================
